import os
import tempfile

from sonotoria import jinja


def populate_repo(project, group):
    with tempfile.TemporaryDirectory() as td:
        project_path = f'{td}/{project.config["name"]}'
        src_path = f'{td}/src'
        os.system(f'git clone https://gitlab.com/kaeraspace/{group.group.name}/{project.config["name"]}.git {project_path}') # nosec B605
        os.system(f'git clone {project.config["src"]} {src_path}') # nosec B605

        if len(project.project.branches.list()) > 0:
            print('No need to populate repo. Skipping.')
            return

        jinja.template_folder(
            src_path,
            project_path,
            context={
                **group.config.get('vars', {}),
                **project.config.get('vars', {}),
                'project': project.config
            }
        )

        os.system(' && '.join([ # nosec B605
            f'cd {project_path}',
            'git add .',
            'git commit -m ":sparkles: Kaera Init"',
            'git push --set-upstream origin master'
        ]))



from contextlib import contextmanager

from .population import populate_repo
from .gitlab_handler import GitlabHandler
from .default import LABELS, BOARDS


def create_full_project(config):
    handler = GitlabHandler(config['token'])

    with ensure(f'group {config["group"]["name"]} is present'):
        group = handler.create_group(config['group'])

    for project_config in config['projects']:
        with ensure(f'project {project_config["name"]} is present'):
            create_repo(project_config, group)


def create_repo(project_config, group):
    with ensure('repo is created'):
        project = group.create_project(project_config)

    with ensure('repo is populated'):
        populate_repo(project, group)

    with ensure('repo branches are protected'):
        project.protect_branches()

    with ensure('repo config is set'):
        project.set_config()

    if not project_config.get('skip_labels', False):
        with ensure('labels are present'):
            project.create_labels(project_config.get('labels', LABELS))
    else:
        other('Skipping labels...')

    if not project_config.get('skip_boards', False):
        with ensure('boards are present'):
            project.create_boards(project_config.get('boards', BOARDS))
    else:
        other('Skipping boards...')


@contextmanager
def ensure(str_):
    print(f'\u001b[33mEnsure {str_}...\u001b[39m')
    yield None
    print(f'\u001b[32mSuccessfully ensured {str_}!\u001b[39m\n')


def other(str_):
    print(f'\u001b[34m{str_}\u001b[39m')



import os
import re
from setuptools import setup

branch = os.environ.get('CI_COMMIT_BRANCH', '')
version = branch if re.match(r'\d+\.\d+', branch) else '0.0'

if __name__ == '__main__':
    setup(
        version=version,
        long_description=open('README.md').read() + '\n\n\n' + open('CHANGELOG.md').read(),
        long_description_content_type='text/markdown'
    )



from gitlab import Gitlab
from gitlab.const import MAINTAINER_ACCESS


KAERASPACE_ID = 13844027


class _Project:
    def __init__(self, group, project, config):
        self.group = group
        self.project = project
        self.config = config
        self.labels = {}

    def protect_branches(self):
        if len(self.project.protectedbranches.list()) > 1:
            return
        self.project.protectedbranches.get('master').delete()
        self.project.protectedbranches.create({
            'name': 'master',
            'merge_access_level': MAINTAINER_ACCESS,
            'push_access_level': MAINTAINER_ACCESS,
            'allow_force_push': True
        })
        self.project.protectedbranches.create({
            'name': '*.*',
            'merge_access_level': MAINTAINER_ACCESS,
            'push_access_level': MAINTAINER_ACCESS,
            'allow_force_push': False
        })

    def create_labels(self, labels):
        self.labels = {
            name: self.create_label(name, color)
            for name, color in labels.items()
        }

    def create_boards(self, boards):
        for name, label_names in boards.items():
            self.create_board(name, self.labels, label_names)

    def create_label(self, name, color):
        label = find_by_name(self.project.labels.list(), name)
        if not label:
            return self.project.labels.create({'name': name, 'color': color})
        label.color = color
        label.save()
        return label

    def create_board(self, name, labels, label_names):
        board = find_by_name(self.project.boards.list(), name)
        if not board:
            board = self.project.boards.create({'name': name})
        for label_name in label_names:
            create_list(board, labels[label_name].id)


class _Group: # pylint: disable=too-few-public-methods
    def __init__(self, group, gl, config):
        self.group = group
        self.gl = gl
        self.config = config

    def create_project(self, config):
        config['name'] = config['name'].lower()
        project = find_by_name(self.group.projects.list(all=True, owned=True), config['name'].capitalize())
        if project:
            project = self.gl.projects.get(project.attributes['id'])
        else:
            project = self.gl.projects.create({
                'name': config['name'].capitalize(),
                'namespace_id': self.group.id,
                'merge_method': 'ff'
            })
        update_resource(project, config.get('desc', ''), config.get('img', None))
        return _Project(self.group, project, config)


class GitlabHandler: # pylint: disable=too-few-public-methods
    def __init__(self, token):
        self.gl = Gitlab('https://gitlab.com', private_token=token)
        self.gl.auth()

    def create_group(self, config):
        group = find_by_name(self.gl.groups.list(owned=True), config['name'])
        if not group:
            group = self.gl.groups.create({
                'name': config['name'],
                'path': config['name'].replace(' ', ''),
                'parent_id': KAERASPACE_ID
            })
        update_resource(group, config.get('desc', ''), config.get('img', None))
        return _Group(group, self.gl, config)


def update_resource(resource, desc, img):
    resource.description = desc
    if img is not None:
        with open(img, 'rb') as img_file:
            resource.avatar = img_file
            resource.save()
            return
    resource.save()


def create_list(board, label_id):
    list_ = find(board.lists.list(), lambda lst: lst.label['id'] == label_id)
    if not list_:
        board.lists.create({'label_id': label_id})


def find_by_name(lst, name):
    return next((el for el in lst if el.name == name), None)


def find(lst, cond):
    return next((el for el in lst if cond(el)), None)


def fuck(project):
    main = project.branches.get('main')
    master = project.branches.create({'branch': 'master', 'ref': 'main'})
    main.unprotect()
    master.protect()
    main.delete()
